@@ -3,6 +3,7 @@
  *
  * Copyright (C) 2003 Neil Brown <neilb@cse.unsw.edu.au>
  * Copyright (C) 2010-2013 Neil Brown <neilb@suse.de>
+ * Copyright (C) 2014-2020 Neil Brown <neil@brown.name>
  *
  *
  *    This program is free software; you can redistribute it and/or modify
@@ -19,7 +20,7 @@
  *    along with this program.
  *
  *    Author: Neil Brown
- *    Email: <neilb@suse.de>
+ *    Email: <neil@brown.name>
  */
 
 /*
@@ -29,10 +30,10 @@
 #include "wiggle.h"
 
 #ifndef VERSION
-#define VERSION "1.1"
+#define VERSION "1.3"
 #endif
 #ifndef VERS_DATE
-#define VERS_DATE "2017-09-28"
+#define VERS_DATE "2020-10-03"
 #endif
 
 char Version[] = "wiggle " VERSION " " VERS_DATE " GPL-2+ http://neil.brown.name/wiggle/\n";
@@ -58,8 +59,11 @@ struct option long_options[] = {
 	{"no-ignore",	0, 0, 'i'},
 	{"show-wiggles",0, 0, 'W'},
 	{"ignore-blanks",0,0, 'b'},
-	{"self-test",   0, 0, SELF_TEST},
+	{"no-backup",	0, 0, NO_BACKUP },
+	{"self-test",	0, 0, SELF_TEST},
 	{"report-wiggles", 0, 0, REPORT_WIGGLES},
+	{"non-space",	0, 0, NON_SPACE},
+	{"shortest",	0, 0, SHORTEST},
 	{0, 0, 0, 0}
 };
 
@@ -83,6 +87,7 @@ char Help[] =  "\n"
 "\n"
 "   --words     -w    : word-wise diff and merge.\n"
 "   --lines     -l    : line-wise diff and merge.\n"
+"   --non-space       : words are separated by spaces.\n"
 "\n"
 "   --patch     -p    : treat last file as a patch file.\n"
 "   -1  -2  -3        : select which component of patch or merge to use.\n"
@@ -96,6 +101,7 @@ char Help[] =  "\n"
 "   --quiet     -q    : don't print un-necessary messages.\n"
 "\n"
 "   --replace   -r    : replace first file with result of merger.\n"
+"   --no-backup       : Never save original file (as name.porig).\n"
 "\n"
 "   --strip=    -p    : number of path components to strip from file names.\n"
 "\n"
@@ -119,7 +125,7 @@ char HelpExtract[] = "\n"
 char HelpDiff[] = "\n"
 "wiggle --diff [-wl] [-p12] [-R]  file-or-patch [file-or-patch]\n"
 "\n"
-"The diff function will report the differencs and similarities between\n"
+"The diff function will report the differences and similarities between\n"
 "two files in a format similar to 'diff -u'.  With --word mode\n"
 "(the default) word-wise differences are displayed on lines starting\n"
 "with a '|'.  With --line mode, only whole lines are considered\n"
