@@ -2,6 +2,7 @@
  * wiggle - apply rejected patches
  *
  * Copyright (C) 2003-2013 Neil Brown <neilb@suse.de>
+ * Copyright (C) 2014-2020 Neil Brown <neil@brown.name>
  *
  *
  *    This program is free software; you can redistribute it and/or modify
@@ -18,7 +19,7 @@
  *    along with this program.
  *
  *    Author: Neil Brown
- *    Email: <neilb@suse.de>
+ *    Email: <neil@brown.name>
  */
 
 /*
@@ -48,7 +49,7 @@
 
 #include "ccan/hash/hash.h"
 
-static int split_internal(char *start, char *end, int type,
+static int wiggle_split_internal(char *start, char *end, int type,
 			  struct elmnt *list)
 {
 	int cnt = 0;
@@ -58,7 +59,7 @@ static int split_internal(char *start, char *end, int type,
 		char *cp2;
 		int prefix = 0;
 
-		if (type == (ByWord | IgnoreBlanks))
+		if ((type & ByWord) && (type & IgnoreBlanks))
 			while (cp < end &&
 			       (*cp == ' ' || *cp == '\t')) {
 				prefix++;
@@ -79,24 +80,28 @@ static int split_internal(char *start, char *end, int type,
 					cp++;
 				break;
 			case ByWord:
-				if (isalnum(*cp) || *cp == '_') {
-					do
-						cp++;
-					while (cp < end
-					       && (isalnum(*cp)
-						   || *cp == '_'));
-				} else if (*cp == ' ' || *cp == '\t') {
+				if (*cp == ' ' || *cp == '\t') {
 					do
 						cp++;
 					while (cp < end
 					       && (*cp == ' '
 						   || *cp == '\t'));
+				} else if ((type & WholeWord) ||
+					   isalnum(*cp) || *cp == '_') {
+					do
+						cp++;
+					while (cp < end
+					       && (((type & WholeWord)
+						    && *cp != ' ' && *cp != '\t'
+						    && *cp != '\n')
+						   || isalnum(*cp)
+						   || *cp == '_'));
 				} else
 					cp++;
 				break;
 			}
 		cp2 = cp;
-		if (type == (ByWord | IgnoreBlanks) &&
+		if ((type & ByWord) && (type & IgnoreBlanks) &&
 		    *start && *start != '\n')
 			while (cp2 < end &&
 			       (*cp2 == ' ' || *cp2 == '\t' || *cp2 == '\n')) {
@@ -121,7 +126,7 @@ static int split_internal(char *start, char *end, int type,
 	return cnt;
 }
 
-struct file split_stream(struct stream s, int type)
+struct file wiggle_split_stream(struct stream s, int type)
 {
 	int cnt;
 	struct file f;
@@ -135,9 +140,9 @@ struct file split_stream(struct stream s, int type)
 	end = s.body+s.len;
 	c = s.body;
 
-	cnt = split_internal(c, end, type, NULL);
-	f.list = xmalloc(cnt*sizeof(struct elmnt));
+	cnt = wiggle_split_internal(c, end, type, NULL);
+	f.list = wiggle_xmalloc(cnt*sizeof(struct elmnt));
 
-	f.elcnt = split_internal(c, end, type, f.list);
+	f.elcnt = wiggle_split_internal(c, end, type, f.list);
 	return f;
 }
