@@ -3,6 +3,7 @@
  *
  * Copyright (C) 2003 Neil Brown <neilb@cse.unsw.edu.au>
  * Copyright (C) 2010-2013 Neil Brown <neilb@suse.de>
+ * Copyright (C) 2014-2020 Neil Brown <neil@brown.name>
  *
  *
  *    This program is free software; you can redistribute it and/or modify
@@ -19,7 +20,7 @@
  *    along with this program.
  *
  *    Author: Neil Brown
- *    Email: <neilb@suse.de>
+ *    Email: <neil@brown.name>
  */
 
 /*
@@ -90,51 +91,6 @@
 #include	<ctype.h>
 #include	<sys/stat.h>
 
-char *Cmd = "wiggle";
-int do_trace = 0;
-
-void die(char *reason)
-{
-	fprintf(stderr, "%s: fatal error: %s failure\n", Cmd, reason);
-	exit(3);
-}
-
-void check_dir(char *name, int fd)
-{
-	struct stat st;
-	if (fstat(fd, &st) != 0) {
-		fprintf(stderr, "%s: fatal: %s is strange\n", Cmd, name);
-		exit(3);
-	}
-	if (S_ISDIR(st.st_mode)) {
-		fprintf(stderr, "%s: %s is a directory\n", Cmd, name);
-		exit(3);
-	}
-}
-
-void *xmalloc(int size)
-{
-	void *rv = malloc(size);
-	if (size && !rv) {
-		char *msg = "Failed to allocate memory - aborting\n";
-		write(2, msg, strlen(msg));
-		exit(3);
-	}
-	return rv;
-}
-
-void printword(FILE *f, struct elmnt e)
-{
-	if (e.start[0])
-		fprintf(f, "%.*s", e.plen + e.prefix,
-			e.start - e.prefix);
-	else {
-		int a, b, c;
-		sscanf(e.start+1, "%d %d %d", &a, &b, &c);
-		fprintf(f, "*** %d,%d **** %d%s", b, c, a, e.start+18);
-	}
-}
-
 static void printsep(struct elmnt e1, struct elmnt e2)
 {
 	int a, b, c, d, e, f;
@@ -152,39 +108,39 @@ static int extract(int argc, char *argv[], int ispatch, int which)
 
 	if (argc == 0) {
 		fprintf(stderr,
-			"%s: no file given for --extract\n", Cmd);
+			"%s: no file given for --extract\n", wiggle_Cmd);
 		return 2;
 	}
 	if (argc > 1) {
 		fprintf(stderr,
-			"%s: only give one file for --extract\n", Cmd);
+			"%s: only give one file for --extract\n", wiggle_Cmd);
 		return 2;
 	}
-	f = load_file(argv[0]);
+	f = wiggle_load_file(argv[0]);
 	if (f.body == NULL) {
 		fprintf(stderr,
-			"%s: cannot load file '%s' - %s\n", Cmd,
+			"%s: cannot load file '%s' - %s\n", wiggle_Cmd,
 			argv[0], strerror(errno));
 		return 2;
 	}
 	if (ispatch) {
-		if (split_patch(f, &flist[0], &flist[1]) == 0) {
+		if (wiggle_split_patch(f, &flist[0], &flist[1]) == 0) {
 			fprintf(stderr,
-				"%s: No chunk found in patch: %s\n", Cmd,
+				"%s: No chunk found in patch: %s\n", wiggle_Cmd,
 				argv[0]);
 			return 0;
 		}
 	} else {
-		if (!split_merge(f, &flist[0], &flist[1], &flist[2])) {
+		if (!wiggle_split_merge(f, &flist[0], &flist[1], &flist[2])) {
 			fprintf(stderr,
-				"%s: merge file %s looks bad.\n", Cmd,
+				"%s: merge file %s looks bad.\n", wiggle_Cmd,
 				argv[0]);
 			return 2;
 		}
 	}
 	if (flist[which-'1'].body == NULL) {
 		fprintf(stderr,
-			"%s: %s has no -%c component.\n", Cmd,
+			"%s: %s has no -%c component.\n", wiggle_Cmd,
 			argv[0], which);
 		return 2;
 	} else {
@@ -205,27 +161,27 @@ static int do_diff_lines(struct file fl[2], struct csl *csl)
 		if (a < csl->a) {
 			if (fl[0].list[a].start[0]) {
 				printf("-");
-				printword(stdout,
-					  fl[0].list[a]);
+				wiggle_printword(stdout,
+						 fl[0].list[a]);
 			}
 			a++;
-			exit_status++;
+			exit_status = 1;
 		} else if (b < csl->b) {
 			if (fl[1].list[b].start[0]) {
 				printf("+");
-				printword(stdout,
-					  fl[1].list[b]);
+				wiggle_printword(stdout,
+						 fl[1].list[b]);
 			}
 			b++;
-			exit_status++;
+			exit_status = 1;
 		} else {
 			if (fl[0].list[a].start[0] == '\0')
 				printsep(fl[0].list[a],
 					 fl[1].list[b]);
 			else {
 				printf(" ");
-				printword(stdout,
-					  fl[0].list[a]);
+				wiggle_printword(stdout,
+						 fl[0].list[a]);
 			}
 			a++;
 			b++;
@@ -239,12 +195,12 @@ static int do_diff_lines(struct file fl[2], struct csl *csl)
 static int do_diff_words(struct file fl[2], struct csl *csl)
 {
 	int a, b;
-	int exit_status  = 0;
+	int exit_status = 0;
 	int sol = 1; /* start of line */
 	a = b = 0;
 	while (a < fl[0].elcnt || b < fl[1].elcnt) {
 		if (a < csl->a) {
-			exit_status++;
+			exit_status = 1;
 			if (sol) {
 				int a1;
 				/* If we remove a
@@ -260,7 +216,7 @@ static int do_diff_words(struct file fl[2], struct csl *csl)
 				if (sol) {
 					printf("-");
 					for (; a < csl->a ; a++) {
-						printword(stdout, fl[0].list[a]);
+						wiggle_printword(stdout, fl[0].list[a]);
 						if (ends_line(fl[0].list[a])) {
 							a++;
 							break;
@@ -274,7 +230,7 @@ static int do_diff_words(struct file fl[2], struct csl *csl)
 				do {
 					if (sol)
 						printf("|");
-					printword(stdout, fl[0].list[a]);
+					wiggle_printword(stdout, fl[0].list[a]);
 					sol = ends_line(fl[0].list[a]);
 					a++;
 				} while (a < csl->a);
@@ -282,7 +238,7 @@ static int do_diff_words(struct file fl[2], struct csl *csl)
 				sol = 0;
 			}
 		} else if (b < csl->b) {
-			exit_status++;
+			exit_status = 1;
 			if (sol) {
 				int b1;
 				sol = 0;
@@ -294,7 +250,7 @@ static int do_diff_words(struct file fl[2], struct csl *csl)
 				if (sol) {
 					printf("+");
 					for (; b < csl->b ; b++) {
-						printword(stdout, fl[1].list[b]);
+						wiggle_printword(stdout, fl[1].list[b]);
 						if (ends_line(fl[1].list[b])) {
 							b++;
 							break;
@@ -308,7 +264,7 @@ static int do_diff_words(struct file fl[2], struct csl *csl)
 				do {
 					if (sol)
 						printf("|");
-					printword(stdout, fl[1].list[b]);
+					wiggle_printword(stdout, fl[1].list[b]);
 					sol = ends_line(fl[1].list[b]);
 					b++;
 				} while (b < csl->b);
@@ -326,7 +282,7 @@ static int do_diff_words(struct file fl[2], struct csl *csl)
 					if (fl[0].list[a].start[0]) {
 						printf(" ");
 						for (; a < csl->a+csl->len; a++, b++) {
-							printword(stdout, fl[0].list[a]);
+							wiggle_printword(stdout, fl[0].list[a]);
 							if (ends_line(fl[0].list[a])) {
 								a++, b++;
 								break;
@@ -340,7 +296,7 @@ static int do_diff_words(struct file fl[2], struct csl *csl)
 					printf("|");
 			}
 			if (!sol) {
-				printword(stdout, fl[0].list[a]);
+				wiggle_printword(stdout, fl[0].list[a]);
 				if (ends_line(fl[0].list[a]))
 					sol = 1;
 				a++;
@@ -354,7 +310,7 @@ static int do_diff_words(struct file fl[2], struct csl *csl)
 }
 
 static int do_diff(int argc, char *argv[], int obj, int ispatch,
-		   int which, int reverse)
+		   int which, int reverse, int shortest)
 {
 	/* create a diff (line or char) of two streams */
 	struct stream f, flist[3];
@@ -365,75 +321,83 @@ static int do_diff(int argc, char *argv[], int obj, int ispatch,
 
 	switch (argc) {
 	case 0:
-		fprintf(stderr, "%s: no file given for --diff\n", Cmd);
+		fprintf(stderr, "%s: no file given for --diff\n", wiggle_Cmd);
 		return 2;
 	case 1:
-		f = load_file(argv[0]);
+		f = wiggle_load_file(argv[0]);
 		if (f.body == NULL) {
 			fprintf(stderr,
-				"%s: cannot load file '%s' - %s\n", Cmd,
+				"%s: cannot load file '%s' - %s\n", wiggle_Cmd,
 				argv[0], strerror(errno));
 			return 2;
 		}
 		chunks1 = chunks2 =
-			split_patch(f, &flist[0], &flist[1]);
+			wiggle_split_patch(f, &flist[0], &flist[1]);
 		if (!flist[0].body || !flist[1].body) {
 			fprintf(stderr,
-				"%s: couldn't parse patch %s\n", Cmd,
+				"%s: couldn't parse patch %s\n", wiggle_Cmd,
 				argv[0]);
 			return 2;
 		}
 		break;
 	case 2:
-		flist[0] = load_file(argv[0]);
+		flist[0] = wiggle_load_file(argv[0]);
 		if (flist[0].body == NULL) {
 			fprintf(stderr,
-				"%s: cannot load file '%s' - %s\n", Cmd,
+				"%s: cannot load file '%s' - %s\n", wiggle_Cmd,
 				argv[0], strerror(errno));
 			return 2;
 		}
 		if (ispatch) {
-			f = load_file(argv[1]);
+			f = wiggle_load_file(argv[1]);
 			if (f.body == NULL) {
 				fprintf(stderr,
-					"%s: cannot load patch '%s' - %s\n", Cmd,
+					"%s: cannot load patch '%s' - %s\n", wiggle_Cmd,
 					argv[1], strerror(errno));
 				return 2;
 			}
 			if (which == '2')
 				chunks2 = chunks3 =
-					split_patch(f, &flist[2],
-						    &flist[1]);
+					wiggle_split_patch(f, &flist[2],
+							   &flist[1]);
 			else
 				chunks2 = chunks3 =
-					split_patch(f, &flist[1],
-						    &flist[2]);
+					wiggle_split_patch(f, &flist[1],
+							   &flist[2]);
 
 		} else
-			flist[1] = load_file(argv[1]);
+			flist[1] = wiggle_load_file(argv[1]);
 		if (flist[1].body == NULL) {
 			fprintf(stderr,
-				"%s: cannot load file '%s' - %s\n", Cmd,
+				"%s: cannot load file '%s' - %s\n", wiggle_Cmd,
 				argv[1], strerror(errno));
 			return 2;
 		}
 		break;
 	default:
 		fprintf(stderr,
-			"%s: too many files given for --diff\n", Cmd);
+			"%s: too many files given for --diff\n", wiggle_Cmd);
 		return 2;
 	}
 	if (reverse) {
-		f = flist[1];
-		flist[1] = flist[2];
-		flist[2] = f;
+		f = flist[0];
+		flist[0] = flist[1];
+		flist[1] = f;
+	}
+	fl[0] = wiggle_split_stream(flist[0], obj);
+	fl[1] = wiggle_split_stream(flist[1], obj);
+	if (!(obj & WholeWord) && fl[0].elcnt > 50000 && fl[1].elcnt > 50000) {
+		/* Too big - use fewer words if possible */
+		free(fl[0].list);
+		free(fl[1].list);
+		obj |= WholeWord;
+		fl[0] = wiggle_split_stream(flist[0], obj);
+		fl[1] = wiggle_split_stream(flist[1], obj);
 	}
-	fl[0] = split_stream(flist[0], obj);
-	fl[1] = split_stream(flist[1], obj);
 	if (chunks2 && !chunks1)
-		csl = pdiff(fl[0], fl[1], chunks2);
+		csl = wiggle_pdiff(fl[0], fl[1], chunks2);
 	else
-		csl = diff_patch(fl[0], fl[1]);
+		csl = wiggle_diff_patch(fl[0], fl[1], shortest);
 	if ((obj & ByMask) == ByLine) {
 		if (!chunks1)
 			printf("@@ -1,%d +1,%d @@\n",
@@ -460,7 +424,7 @@ static int do_diff(int argc, char *argv[], int obj, int ispatch,
 static int do_merge(int argc, char *argv[], int obj, int blanks,
 		    int reverse, int replace, char *outfilename,
 		    int ignore, int show_wiggles,
-		    int quiet)
+		    int quiet, int shortest)
 {
 	/* merge three files, A B C, so changed between B and C get made to A
 	 */
@@ -475,16 +439,16 @@ static int do_merge(int argc, char *argv[], int obj, int blanks,
 
 	switch (argc) {
 	case 0:
-		fprintf(stderr, "%s: no files given for --merge\n", Cmd);
+		fprintf(stderr, "%s: no files given for --merge\n", wiggle_Cmd);
 		return 2;
 	case 3:
 	case 2:
 	case 1:
 		for (i = 0; i < argc; i++) {
-			flist[i] = load_file(argv[i]);
+			flist[i] = wiggle_load_file(argv[i]);
 			if (flist[i].body == NULL) {
 				fprintf(stderr, "%s: cannot load file '%s' - %s\n",
-					Cmd,
+					wiggle_Cmd,
 					argv[i], strerror(errno));
 				return 2;
 			}
@@ -492,22 +456,22 @@ static int do_merge(int argc, char *argv[], int obj, int blanks,
 		break;
 	default:
 		fprintf(stderr, "%s: too many files given for --merge\n",
-			Cmd);
+			wiggle_Cmd);
 		return 2;
 	}
 	switch (argc) {
 	case 1: /* a merge file */
 		f = flist[0];
-		if (!split_merge(f, &flist[0], &flist[1], &flist[2])) {
+		if (!wiggle_split_merge(f, &flist[0], &flist[1], &flist[2])) {
 			fprintf(stderr, "%s: merge file %s looks bad.\n",
-				Cmd,
+				wiggle_Cmd,
 				argv[0]);
 			return 2;
 		}
 		break;
 	case 2: /* a file and a patch */
 		f = flist[1];
-		chunks2 = chunks3 = split_patch(f, &flist[1], &flist[2]);
+		chunks2 = chunks3 = wiggle_split_patch(f, &flist[1], &flist[2]);
 		break;
 	case 3: /* three separate files */
 		break;
@@ -520,7 +484,7 @@ static int do_merge(int argc, char *argv[], int obj, int blanks,
 
 	for (i = 0; i < 3; i++) {
 		if (flist[i].body == NULL) {
-			fprintf(stderr, "%s: file %d missing\n", Cmd, i);
+			fprintf(stderr, "%s: file %d missing\n", wiggle_Cmd, i);
 			return 2;
 		}
 	}
@@ -528,21 +492,23 @@ static int do_merge(int argc, char *argv[], int obj, int blanks,
 		outfile = fopen(outfilename, "w");
 		if (!outfile) {
 			fprintf(stderr, "%s: could not create %s\n",
-				Cmd, outfilename);
+				wiggle_Cmd, outfilename);
 			return 2;
 		}
 	} else if (replace) {
 		int fd;
-		replacename = xmalloc(strlen(argv[0]) + 20);
-		orignew = xmalloc(strlen(argv[0]) + 20);
+		replacename = wiggle_xmalloc(strlen(argv[0]) + 20);
+		orignew = wiggle_xmalloc(strlen(argv[0]) + 20);
 		strcpy(replacename, argv[0]);
 		strcpy(orignew, argv[0]);
 		strcat(orignew, ".porig");
 		if (open(orignew, O_RDONLY) >= 0 ||
 		    errno != ENOENT) {
 			fprintf(stderr, "%s: %s already exists\n",
-				Cmd,
+				wiggle_Cmd,
 				orignew);
+			free(replacename);
+			free(orignew);
 			return 2;
 		}
 		strcat(replacename, "XXXXXX");
@@ -550,8 +516,10 @@ static int do_merge(int argc, char *argv[], int obj, int blanks,
 		if (fd == -1) {
 			fprintf(stderr,
 				"%s: could not create temporary file for %s\n",
-				Cmd,
+				wiggle_Cmd,
 				replacename);
+			free(replacename);
+			free(orignew);
 			return 2;
 		}
 		outfile = fdopen(fd, "w");
@@ -561,19 +529,31 @@ static int do_merge(int argc, char *argv[], int obj, int blanks,
 		blanks |= ByLine;
 	else
 		blanks |= ByWord;
-	fl[0] = split_stream(flist[0], blanks);
-	fl[1] = split_stream(flist[1], blanks);
-	fl[2] = split_stream(flist[2], blanks);
+	fl[0] = wiggle_split_stream(flist[0], blanks);
+	fl[1] = wiggle_split_stream(flist[1], blanks);
+	fl[2] = wiggle_split_stream(flist[2], blanks);
+	if (!(blanks & WholeWord) &&
+	    fl[1].elcnt > 50000 &&
+	    (fl[0].elcnt > 50000 || fl[2].elcnt > 50000)) {
+		/* Too many words */
+		free(fl[0].list);
+		free(fl[1].list);
+		free(fl[2].list);
+		blanks |= WholeWord;
+		fl[0] = wiggle_split_stream(flist[0], blanks);
+		fl[1] = wiggle_split_stream(flist[1], blanks);
+		fl[2] = wiggle_split_stream(flist[2], blanks);
+	}
 
 	if (chunks2 && !chunks1)
-		csl1 = pdiff(fl[0], fl[1], chunks2);
+		csl1 = wiggle_pdiff(fl[0], fl[1], chunks2);
 	else
-		csl1 = diff(fl[0], fl[1]);
-	csl2 = diff_patch(fl[1], fl[2]);
+		csl1 = wiggle_diff(fl[0], fl[1], shortest);
+	csl2 = wiggle_diff_patch(fl[1], fl[2], shortest);
 
-	ci = make_merger(fl[0], fl[1], fl[2], csl1, csl2,
+	ci = wiggle_make_merger(fl[0], fl[1], fl[2], csl1, csl2,
 			 obj == 'w', ignore, show_wiggles > 1);
-	print_merge(outfile, &fl[0], &fl[1], &fl[2],
+	wiggle_print_merge(outfile, &fl[0], &fl[1], &fl[2],
 		    obj == 'w', ci.merger, NULL, 0, 0);
 	if (!quiet && ci.conflicts)
 		fprintf(stderr,
@@ -589,6 +569,24 @@ static int do_merge(int argc, char *argv[], int obj, int blanks,
 	if (outfilename)
 		fclose(outfile);
 	else if (replace) {
+		struct stat statbuf;
+
+		if (stat(argv[0], &statbuf) != 0) {
+			fprintf(stderr,
+				"%s: failed to stat original file. - %s\n",
+				wiggle_Cmd, strerror(errno));
+			free(replacename);
+			free(orignew);
+			return 2;
+		}
+		if (fchmod(fileno(outfile), statbuf.st_mode) != 0) {
+			fprintf(stderr,
+				"%s: failed to change permission of new file. - %s\n",
+				wiggle_Cmd, strerror(errno));
+			free(replacename);
+			free(orignew);
+			return 2;
+		}
 		fclose(outfile);
 		if (rename(argv[0], orignew) == 0 &&
 		    rename(replacename, argv[0]) == 0)
@@ -596,10 +594,14 @@ static int do_merge(int argc, char *argv[], int obj, int blanks,
 		else {
 			fprintf(stderr,
 				"%s: failed to move new file into place.\n",
-				Cmd);
+				wiggle_Cmd);
+			free(replacename);
+			free(orignew);
 			return 2;
 		}
 	}
+	free(replacename);
+	free(orignew);
 	if (show_wiggles)
 		return ci.conflicts + ci.wiggles > 0;
 	else
@@ -609,7 +611,7 @@ static int do_merge(int argc, char *argv[], int obj, int blanks,
 static int multi_merge(int argc, char *argv[], int obj, int blanks,
 		       int reverse, int ignore, int show_wiggles,
 		       int replace, int strip,
-		       int quiet)
+		       int quiet, int shortest)
 {
 	FILE *f;
 	char *filename;
@@ -621,27 +623,27 @@ static int multi_merge(int argc, char *argv[], int obj, int blanks,
 	if (!replace) {
 		fprintf(stderr,
 			"%s: -p in merge mode requires -r\n",
-			Cmd);
+			wiggle_Cmd);
 		return 2;
 	}
 	if (argc != 1) {
 		fprintf(stderr,
 			"%s: -p in merge mode requires exactly one file\n",
-			Cmd);
+			wiggle_Cmd);
 		return 2;
 	}
 	filename = argv[0];
 	f = fopen(filename, "r");
 	if (!f) {
 		fprintf(stderr, "%s: cannot open %s\n",
-			Cmd, filename);
+			wiggle_Cmd, filename);
 		return 2;
 	}
-	check_dir(filename, fileno(f));
-	pl = parse_patch(f, NULL, &num_patches);
+	wiggle_check_dir(filename, fileno(f));
+	pl = wiggle_parse_patch(f, NULL, &num_patches);
 	fclose(f);
-	if (set_prefix(pl, num_patches, strip) == 0) {
-		fprintf(stderr, "%s: aborting\n", Cmd);
+	if (wiggle_set_prefix(pl, num_patches, strip) == 0) {
+		fprintf(stderr, "%s: aborting\n", wiggle_Cmd);
 		return 2;
 	}
 	for (i = 0; i < num_patches; i++) {
@@ -652,7 +654,7 @@ static int multi_merge(int argc, char *argv[], int obj, int blanks,
 		av[0] = pl[i].file;
 		av[1] = name;
 		rv |= do_merge(2, av, obj, blanks, reverse, 1, NULL, ignore,
-			 show_wiggles, quiet);
+			       show_wiggles, quiet, shortest);
 	}
 	return rv;
 }
@@ -664,6 +666,7 @@ int main(int argc, char *argv[])
 	int mode = 0;
 	int obj = 0;
 	int replace = 0;
+	int backup = 1;
 	int which = 0;
 	int ispatch = 0;
 	int reverse = 0;
@@ -671,6 +674,7 @@ int main(int argc, char *argv[])
 	int strip = -1;
 	int exit_status = 0;
 	int ignore = 1;
+	int shortest = 0;
 	int show_wiggles = 0;
 	char *helpmsg;
 	char *trace;
@@ -680,7 +684,7 @@ int main(int argc, char *argv[])
 
 	trace = getenv("WIGGLE_TRACE");
 	if (trace && *trace)
-		do_trace = 1;
+		wiggle_do_trace = 1;
 
 	while ((opt = getopt_long(argc, argv,
 				  short_options, long_options,
@@ -729,9 +733,17 @@ int main(int argc, char *argv[])
 			}
 			fprintf(stderr,
 				"%s: mode is '%c' - cannot set to '%c'\n",
-				Cmd, mode, opt);
+				wiggle_Cmd, mode, opt);
 			exit(2);
 
+		case NON_SPACE:
+			ignore_blanks |= WholeWord;
+			continue;
+
+		case SHORTEST:
+			shortest = 1;
+			continue;
+
 		case 'w':
 		case 'l':
 			if (obj == 0 || obj == opt) {
@@ -739,12 +751,15 @@ int main(int argc, char *argv[])
 				continue;
 			}
 			fprintf(stderr,
-				"%s: cannot select both words and lines.\n", Cmd);
+				"%s: cannot select both words and lines.\n", wiggle_Cmd);
 			exit(2);
 
 		case 'r':
 			replace = 1;
 			continue;
+		case NO_BACKUP:
+			backup = 0;
+			continue;
 		case 'o':
 			outfile = optarg;
 			replace = 1;
@@ -754,7 +769,7 @@ int main(int argc, char *argv[])
 			continue;
 
 		case 'b':
-			ignore_blanks = IgnoreBlanks;
+			ignore_blanks |= IgnoreBlanks;
 			continue;
 
 		case 'i':
@@ -776,7 +791,7 @@ int main(int argc, char *argv[])
 				continue;
 			}
 			fprintf(stderr,
-				"%s: can only select one of -1, -2, -3\n", Cmd);
+				"%s: can only select one of -1, -2, -3\n", wiggle_Cmd);
 			exit(2);
 
 		case 'p': /* 'patch' or 'strip' */
@@ -802,7 +817,7 @@ int main(int argc, char *argv[])
 	if (mode == 'B') {
 		vpatch(argc-optind, argv+optind, ispatch,
 		       strip, reverse, replace, outfile, selftest,
-		       ignore_blanks);
+		       ignore_blanks, backup);
 		/* should not return */
 		exit(1);
 	}
@@ -810,36 +825,36 @@ int main(int argc, char *argv[])
 	if (obj && mode == 'x') {
 		fprintf(stderr,
 			"%s: cannot specify --line or --word with --extract\n",
-			Cmd);
+			wiggle_Cmd);
 		exit(2);
 	}
 	if (mode != 'm' && !obj)
 		obj = 'w';
 	if (ispatch && outfile) {
 		fprintf(stderr, "%s: --output incompatible with --patch\n",
-			Cmd);
+			wiggle_Cmd);
 		exit(2);
 	}
 	if (replace && mode != 'm') {
 		fprintf(stderr,
-			"%s: --replace or --output only allowed with --merge\n", Cmd);
+			"%s: --replace or --output only allowed with --merge\n", wiggle_Cmd);
 		exit(2);
 	}
 	if (mode == 'x' && !which) {
 		fprintf(stderr,
-			"%s: must specify -1, -2 or -3 with --extract\n", Cmd);
+			"%s: must specify -1, -2 or -3 with --extract\n", wiggle_Cmd);
 		exit(2);
 	}
 	if (mode != 'x' && mode != 'd' && which) {
 		fprintf(stderr,
 			"%s: -1, -2 or -3 only allowed with --extract or --diff\n",
-			Cmd);
+			wiggle_Cmd);
 		exit(2);
 	}
 
 	if (ispatch && which == '3') {
 		fprintf(stderr,
-			"%s: cannot extract -3 from a patch.\n", Cmd);
+			"%s: cannot extract -3 from a patch.\n", wiggle_Cmd);
 		exit(2);
 	}
 
@@ -851,7 +866,7 @@ int main(int argc, char *argv[])
 		exit_status = do_diff(argc-optind, argv+optind,
 				      (obj == 'l' ? ByLine : ByWord)
 				      | ignore_blanks,
-				      ispatch, which, reverse);
+				      ispatch, which, reverse, shortest);
 		break;
 	case 'm':
 		if (ispatch)
@@ -861,13 +876,13 @@ int main(int argc, char *argv[])
 						  reverse, ignore,
 						  show_wiggles,
 						  replace, strip,
-						  quiet);
+						  quiet, shortest);
 		else
 			exit_status = do_merge(
 				argc-optind, argv+optind,
 				obj, ignore_blanks, reverse, replace,
 				outfile,
-				ignore, show_wiggles, quiet);
+				ignore, show_wiggles, quiet, shortest);
 		break;
 	}
 	exit(exit_status);
